import 'dart:typed_data';
import 'package:code_id_flutter/code_interfaces/storage/i_storage.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:hive_flutter/hive_flutter.dart';

class Storage implements IStorage {
  late Box _box;

  final HiveInterface hive;

  Storage(this.hive);

  Future openBox(
    String boxName,
  ) async {
    List<int>? hiveKey = await hiveKeys;

    _box = await hive.openBox(
      boxName,
      encryptionCipher: HiveAesCipher(hiveKey),
    );
    debugPrint('====================Open Box=================');
    debugPrint('Database $boxName Open');
    debugPrint('=============================================');
  }

  Future close() async {
    await _box.close();
    return;
  }

  Future<void> putData({required Map<String, dynamic> json}) async {
    try {
      debugPrint('====================Put Data=================');
      debugPrint('$json');
      debugPrint('=============================================');
      await _box.putAll(json);
      debugPrint('================Put Data Success=============');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint('${e.toString()}');
      debugPrint('=============================================');
    }
    return;
  }

  Future<void> putDynamicData(
      {required String key, required dynamic value}) async {
    try {
      debugPrint('====================Put Data=================');
      debugPrint('$value');
      debugPrint('=============================================');
      await _box.put(key, value);
      debugPrint('================Put Data Success=============');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint('${e.toString()}');
      debugPrint('=============================================');
    }
    return;
  }

  Future<void> putListData({required List dataList}) async {
    try {
      debugPrint('====================Put Data=================');
      dataList.map((val) async {
        debugPrint('$val');
        await _box.add(val);
      });
      debugPrint('=============================================');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint('${e.toString()}');
      debugPrint('=============================================');
    }

    return;
  }

  Future<void> putString({required String key, required String value}) async {
    try {
      debugPrint('====================Put Data=================');
      debugPrint('$value');
      debugPrint('=============================================');
      await _box.put(key, value);
      debugPrint('================Put Data Success=============');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint('${e.toString()}');
      debugPrint('=============================================');
    }
    return;
  }

  Future<void> putBool({required String key, required bool value}) async {
    try {
      debugPrint('====================Put Data=================');
      debugPrint('$value');
      debugPrint('=============================================');
      await _box.put(key, value);
      debugPrint('================Put Data Success=============');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint('${e.toString()}');
      debugPrint('=============================================');
    }
    return;
  }

  String? getString({required String key}) {
    debugPrint('====================Get Data=================');
    String? value = _box.get(key);
    debugPrint('$value');
    debugPrint('=============================================');
    return value;
  }

  DateTime? getDate({required String key}) {
    DateTime? date = _box.get(key);
    // box.close();
    return date;
  }

  int? getInt({required String key}) {
    int? value = _box.get(key);

    return value;
  }

  bool getBool({required String key}) {
    bool? value = _box.get(key);
    if (value == null) {
      value = false;
    }
    // box.close();
    return value;
  }

  double? getDouble({required String key}) {
    double? value = _box.get(key);

    return value;
  }

  Map<String, dynamic>? get getData {
    Map<String, dynamic>? value = Map<String, dynamic>.from(_box.toMap());
    print(value);
    return value;
  }

  getDynamicData({required String key}) {
    return _box.get(key);
  }

  List? getListData() {
    final value = _box.toMap();
    print(value);
    List datas = [];
    value.forEach((key, value) {
      datas.add(value);
    });
    // box.close();
    return datas;
  }

  Future<void> deleteData() async {
    await _box.deleteFromDisk();
    return;
  }

  Future<void> deleteString({required String key}) async {
    await _box.delete(key);
    // box.close();
    return;
  }

  Future<List<int>> get hiveKeys async {
    final ss = FlutterSecureStorage();
    String? stringKey = await ss.read(key: 'boxKey');
    List<int> hiveKey;
    if (stringKey != null) {
      hiveKey = stringKey.codeUnits;
    } else {
      hiveKey = Hive.generateSecureKey();
      Uint8List bytes = Uint8List.fromList(hiveKey);
      stringKey = String.fromCharCodes(bytes);
      await ss.write(key: 'boxKey', value: stringKey);
    }
    return hiveKey;
  }

  @override
  bool isStorageOpen() {
    return _box.isOpen;
  }

  @override
  ValueListenable<Box> listenable({List? keys}) => _box.listenable(keys: keys);
}
